/*  Nucleotide Sequence Generator - seq-gen, version 1.05  */
/*  (c) Copyright 1996, Andrew Rambaut & Nick Grassly      */
/*  Department of Zoology, University of Oxford            */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "lik.h"
#include "tree.h"
#include "evolve.h"
#include "gamma.h"
#include "random.h"

char *categories;
double addFreq[4];

/* prototypes */

char SetBase(double *P);
void SetSequence(char *seq, char *source);
void RandomSequence(char *seq);
void MutateSequence(TTree *tree, char *seq, int branchNo);
void EvolveNode(TTree *tree, TNode *anc);

/* functions */

void SetupEvolve()
{
	int i;
	
	categories=CAllocMem(numBases*sizeof(short), "categories", "AllocateEvolve", 0);

	addFreq[A]=freqA;
	addFreq[C]=addFreq[A]+freqC;
	addFreq[G]=addFreq[C]+freqG;
	addFreq[T]=addFreq[G]+freqT;
	
	if (gammaRates) {
		for (i=0; i<numBases; i++)
			categories[i]=(int)(rndu()*numCategories);
	} else if (codonRates) {
		for (i=0; i<numBases; i++)
			categories[i] = codonCats[i % 3];
	} else {
		for (i=0; i<numBases; i++)
			categories[i] = 0;
	}
}

char SetBase(double *P)
{
	char j;
	double r;
	
	r=rndu();
	for (j='\0'; r>(*P) && j<'\3'; j++) P++;
	return j;
}


void RandomSequence(char *seq)
{
	int i;
	char *P;
	
	P=seq;
	for (i=0; i<numBases; i++) {
		*P=SetBase(addFreq);
		P++;
	}
}

void MutateSequence(TTree *tree, char *seq, int branchNo)
{
	int i, cat, offset;
	char *R;
	char *P;
	
	P=seq;
	offset = branchNo * 16;
	R=categories;
	for (i=0; i<numBases; i++) {
		*P=SetBase(tree->transMatrix[*R] + offset + (*P<<2));
		P++;
		R++;
	}
}


void EvolveNode(TTree *tree, TNode *anc)
{
	TNode *des = anc->branch;
	
	memcpy(des->sequence, anc->sequence, numBases);
	MutateSequence(tree, des->sequence, des->branchNo);
	
	if (des->tipNo==-1) {
		EvolveNode(tree, des->next);
		EvolveNode(tree, des->next->next);
	}
}


void EvolveSequences(TTree *tree)
{
	RandomSequence(tree->start->sequence);
	
	EvolveNode(tree, tree->start);
	EvolveNode(tree, tree->start->next);
	EvolveNode(tree, tree->start->next->next);
}


